# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NumberedInterval_to_named_interval_01():

    numbered_interval = pitchtools.NumberedInterval(0)
    named_interval = numbered_interval.to_named_interval(1)
    assert named_interval == pitchtools.NamedInterval('perfect', 1)

    numbered_interval = pitchtools.NumberedInterval(1)
    named_interval = numbered_interval.to_named_interval(1)
    assert named_interval == pitchtools.NamedInterval('augmented', 1)

    numbered_interval = pitchtools.NumberedInterval(0)
    named_interval = numbered_interval.to_named_interval(2)
    assert named_interval == pitchtools.NamedInterval('diminished', 2)

    numbered_interval = pitchtools.NumberedInterval(1)
    named_interval = numbered_interval.to_named_interval(2)
    assert named_interval == pitchtools.NamedInterval('minor', 2)

    numbered_interval = pitchtools.NumberedInterval(2)
    named_interval = numbered_interval.to_named_interval(2)
    assert named_interval == pitchtools.NamedInterval('major', 2)

    numbered_interval = pitchtools.NumberedInterval(3)
    named_interval = numbered_interval.to_named_interval(2)
    assert named_interval == pitchtools.NamedInterval('augmented', 2)


def test_pitchtools_NumberedInterval_to_named_interval_02():

    numbered_interval = pitchtools.NumberedInterval(0)
    named_interval = numbered_interval.to_named_interval(1)
    assert named_interval == pitchtools.NamedInterval('perfect', 1)

    numbered_interval = pitchtools.NumberedInterval(-1)
    named_interval = numbered_interval.to_named_interval(1)
    assert named_interval == pitchtools.NamedInterval('augmented', -1)

    numbered_interval = pitchtools.NumberedInterval(0)
    named_interval = numbered_interval.to_named_interval(2)
    # THE ASCENDING INTERVAL HERE IS WEIRD #
    assert named_interval == pitchtools.NamedInterval('diminished', 2)

    numbered_interval = pitchtools.NumberedInterval(-1)
    named_interval = numbered_interval.to_named_interval(2)
    assert named_interval == pitchtools.NamedInterval('minor', -2)

    numbered_interval = pitchtools.NumberedInterval(-2)
    named_interval = numbered_interval.to_named_interval(2)
    assert named_interval == pitchtools.NamedInterval('major', -2)

    numbered_interval = pitchtools.NumberedInterval(-3)
    named_interval = numbered_interval.to_named_interval(2)
    assert named_interval == pitchtools.NamedInterval('augmented', -2)



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_LilyPondCommand_format_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    command = indicatortools.LilyPondCommand("#(set-accidental-style 'forget)")
    attach(command, staff)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            #(set-accidental-style 'forget)
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )


def test_indicatortools_LilyPondCommand_format_02():

    staff = Staff("c'8 d'8 e'8 f'8")
    command = indicatortools.LilyPondCommand("#(set-accidental-style 'forget)")
    attach(command, staff[1])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            #(set-accidental-style 'forget)
            d'8
            e'8
            f'8
        }
        '''
        )


def test_indicatortools_LilyPondCommand_format_03():
    r'''Barline after leaf.
    '''

    note = Note("c'4")
    command = indicatortools.LilyPondCommand(r'break', 'after')
    attach(command, note)

    assert format(note) == stringtools.normalize(
        r'''
        c'4
        \break
        '''
        )


def test_indicatortools_LilyPondCommand_format_04():
    r'''Barline at container closing.
    '''

    staff = Staff()
    command = indicatortools.LilyPondCommand(r'break')
    attach(command, staff)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \break
        }
        '''
        )


def test_indicatortools_LilyPondCommand_format_05():
    r'''Add a natural harmonic.
    '''

    note = Note("c'4")
    command = indicatortools.LilyPondCommand('flageolet', 'right')
    attach(command, note)
    assert format(note) == "c'4 \\flageolet"


def test_indicatortools_LilyPondCommand_format_06():
    r'''Add then detach natural harmonic.
    '''

    note = Note("c'4")
    command = indicatortools.LilyPondCommand('flageolet', 'right')
    attach(command, note)
    detach(command, note)
    assert format(note) == "c'4"


def test_indicatortools_LilyPondCommand_format_07():

    staff = Staff([Note("c'4")])
    command = indicatortools.LilyPondCommand('compressFullBarRests')
    attach(command, staff[0])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \compressFullBarRests
            c'4
        }
        '''
        )


def test_indicatortools_LilyPondCommand_format_08():

    staff = Staff([Note("c'4")])
    command = indicatortools.LilyPondCommand('expandFullBarRests')
    attach(command, staff[0])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \expandFullBarRests
            c'4
        }
        '''
        )


def test_indicatortools_LilyPondCommand_format_09():
    r'''Voice number can be set on leaves.
    '''

    voice = Voice("c'8 c'8 c'8 c'8")
    command = indicatortools.LilyPondCommand('voiceOne')
    attach(command, voice[0])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \voiceOne
            c'8
            c'8
            c'8
            c'8
        }
        '''
        )


def test_indicatortools_LilyPondCommand_format_10():
    r'''Voice number can be set to 1, 2, 3, 4, or None.
    Anyhing else will throw a ValueError exception.
    '''

    voice = Voice("c'8 c'8 c'8 c'8")
    command = indicatortools.LilyPondCommand('voiceOne')
    attach(command, voice[0])
    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \voiceOne
            c'8
            c'8
            c'8
            c'8
        }
        '''
        )

    detach(command, voice[0])
    command = indicatortools.LilyPondCommand('voiceTwo')
    attach(command, voice[0])
    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \voiceTwo
            c'8
            c'8
            c'8
            c'8
        }
        '''
        )

    detach(command, voice[0])
    command = indicatortools.LilyPondCommand('voiceThree')
    attach(command, voice[0])
    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \voiceThree
            c'8
            c'8
            c'8
            c'8
        }
        '''
        )

    detach(command, voice[0])
    command = indicatortools.LilyPondCommand('voiceFour')
    attach(command, voice[0])
    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \voiceFour
            c'8
            c'8
            c'8
            c'8
        }
        '''
        )

    detach(command, voice[0])
    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8
            c'8
            c'8
            c'8
        }
        '''
        )


def test_indicatortools_LilyPondCommand_format_11():
    r'''Voice number can be set on a Voice container and on one of the
    leaves contained in it.
    '''

    voice = Voice("c'8 c'8 c'8 c'8")
    command = indicatortools.LilyPondCommand('voiceOne')
    attach(command, voice)
    command = indicatortools.LilyPondCommand('voiceTwo')
    attach(command, voice[1])
    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \voiceOne
            c'8
            \voiceTwo
            c'8
            c'8
            c'8
        }
        '''
        )



# -*- coding: utf-8 -*-
# tools first, materials second, segments last:
# makes it possible for materials to import tools;
# makes it possible for segments to import both materials and tools.
from {score_package_name} import tools
from {score_package_name} import materials
from {score_package_name} import segments



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_ScaleDegree_apply_accidental_01():

    degree = tonalanalysistools.ScaleDegree('flat', 2)
    assert degree.apply_accidental('sharp') == tonalanalysistools.ScaleDegree(2)
    assert degree.apply_accidental('ss') == tonalanalysistools.ScaleDegree('sharp', 2)



# -*- coding: utf-8 -*-
import collections
from abjad.tools import sequencetools
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class CompoundOperator(AbjadValueObject):
    r'''Compound operator.

    ..  container:: example

        Rotation followed by transposition:

        ::

            >>> operator = pitchtools.CompoundOperator()
            >>> operator = operator.rotate(n=1, stravinsky=True)
            >>> operator = operator.transpose(n=2)

        ::

            >>> str(operator)
            'T2rs1'

        ::

            >>> pitch_classes = pitchtools.PitchClassSegment([0, 1, 4, 7])
            >>> operator(pitch_classes)
            PitchClassSegment([2, 7, 8, 11])

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_operators',
        '_show_identity_operators',
        )

    _publish_storage_format = True

    ### INITIALIZER ###

    def __init__(self, operators=None, show_identity_operators=None):
        if operators is not None:
            if not isinstance(operators, collections.Sequence):
                operators = (operators,)
            assert len(operators)
            operators = tuple(operators)
        self._operators = operators
        assert isinstance(show_identity_operators, (bool, type(None)))
        self._show_identity_operators = show_identity_operators

    ### SPECIAL METHODS ###

    def __add__(self, operator):
        r'''Composes compound operator and `operator`.

        ..  container:: example

            ::

                >>> operator = pitchtools.CompoundOperator()
                >>> operator = operator.transpose(n=1)
                >>> operator = operator.multiply(n=5)
                >>> str(operator)
                'M5T1'

        ..  container:: example

            ::

                >>> inversion = pitchtools.Inversion()
                >>> retrograde = pitchtools.Retrograde()
                >>> transposition = pitchtools.Transposition(n=1)

            ::

                >>> operator_1 = inversion + retrograde
                >>> str(operator_1)
                'IR'

            ::

                >>> operator_2 = inversion + transposition
                >>> str(operator_2)
                'IT1'

            ::

                >>> operator_3 = operator_1 + operator_2
                >>> str(operator_3)
                'IRIT1'

            ::

                >>> f(operator_3)
                pitchtools.CompoundOperator(
                    operators=[
                        pitchtools.Transposition(
                            n=1,
                            ),
                        pitchtools.Inversion(),
                        pitchtools.Retrograde(),
                        pitchtools.Inversion(),
                        ],
                    )

        Returns new compound operator.
        '''
        operators = list(self.operators)
        if isinstance(operator, type(self)):
            operators[0:0] = operator.operators
        else:
            operators.insert(0, operator)
        result = type(self)()
        for operator in operators:
            result = result._with_operator(operator)
        return result

    def __call__(self, argument):
        r'''Calls compound operator on `argument`.

        ..  container:: example

            ::

                >>> operator = pitchtools.CompoundOperator()
                >>> operator = operator.rotate(n=1, stravinsky=True)
                >>> operator = operator.transpose(n=2)
                >>> str(operator)
                'T2rs1'

            ::

                >>> segment = pitchtools.PitchClassSegment([0, 1, 4, 7])
                >>> show(segment) # doctest: +SKIP

            ::

                >>> transform = operator(segment)
                >>> show(transform) # doctest: +SKIP

            ::

                >>> transform
                PitchClassSegment([2, 7, 8, 11])

        Returns new object with type equal to that of `argument`.
        '''
        if self.operators is None:
            return argument
        for transform in self.operators:
            argument = transform(argument)
        return argument

    def __radd__(self, operator):
        r'''Composes `operator` and compound operator.

        ..  container

            ::

                >>> operator = pitchtools.CompoundOperator()
                >>> operator = operator.transpose(n=1)
                >>> operator = operator.multiply(n=5)
                >>> str(operator)
                'M5T1'

            ::

                >>> retrograde = Retrograde()
                >>> new_operator = retrograde + operator
                >>> str(new_operator)
                'RM5T1'

            ::

                >>> new_operator = operator + retrograde
                >>> str(new_operator)
                'M5T1R'

        Returns new compound operator.
        '''
        operators = list(self.operators)
        if isinstance(operator, type(self)):
            operators.extend(operator.operators)
        else:
            operators.append(operator)
        result = type(self)()
        for operator in operators:
            result = result._with_operator(operator)
        return result

    def __str__(self):
        r'''Gets string representation of compound operator.

        ..  container:: example

            Gets string:

            ::

                >>> operator = pitchtools.CompoundOperator()
                >>> operator = operator.rotate(n=1)
                >>> operator = operator.transpose(n=2)

            ::

                >>> str(operator)
                'T2r1'

        ..  container:: example

            Gets string of empty operator:

            ::

                >>> operator = pitchtools.CompoundOperator()

            ::

                >>> str(operator)
                ''

        Returns string.
        '''
        result = []
        operators = self.operators or []
        for operator in reversed(operators):
            if operator._is_identity_operator():
                if self.show_identity_operators:
                    result.append(str(operator))
            else:
                result.append(str(operator))
        result = ''.join(result)
        return result

    ### PRIVATE METHODS ###

    @staticmethod
    def _compose_operators(operator_1, operator_2):
        if isinstance(operator_1, CompoundOperator):
            result = operator_1.__add__(operator_2)
        elif isinstance(operator_2, CompoundOperator):
            result = operator_2.__radd__(operator_1)
        else:
            result = CompoundOperator()
            result = result._with_operator(operator_2)
            result = result._with_operator(operator_1)
        return result

    def _get_markup(self, direction=None):
        from abjad.tools import markuptools
        markups = []
        operators = self.operators or []
        for operator in operators:
            markup = operator._get_markup(direction=direction)
            if (self.show_identity_operators or
                not operator._is_identity_operator()):
                markups.append(markup)
        if len(markups) == 0:
            return
        elif len(markups) == 1:
            markup = markups[0]
        else:
            markup = markuptools.Markup.concat(markups, direction=direction)
        return markup
        
    def _with_operator(self, operator):
        operators = self.operators or []
        operators = operators + [operator]
        return type(self)(
            operators,
            show_identity_operators=self.show_identity_operators,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def operators(self):
        r'''Gets operators.

        ..  container:: example

            Gets operators:

            ::

                >>> operator = pitchtools.CompoundOperator()
                >>> operator = operator.rotate(n=1)
                >>> operator = operator.transpose(n=2)

            ::

                >>> for operator_ in operator.operators:
                ...     operator_
                ...
                Rotation(n=1)
                Transposition(n=2)

        Returns list of operators.
        '''
        if self._operators is not None:
            return list(self._operators)

    @property
    def show_identity_operators(self):
        r'''Is true when string representation of operator should show identity
        operators. Otherwise false.

        ..  container:: example

            Does not show identity operators:

            ::

                >>> operator = pitchtools.CompoundOperator()
                >>> operator = operator.transpose(n=0)
                >>> operator = operator.multiply(n=5)

            ::

                >>> str(operator)
                'M5'

        ..  container:: example

            Shows identity operators:

            ::

                >>> operator = pitchtools.CompoundOperator(
                ...     show_identity_operators=True,
                ...     )
                >>> operator = operator.transpose(n=0)
                >>> operator = operator.multiply(n=5)

            ::

                >>> str(operator)
                'M5T0'

        Defaults to none.

        Set to true, false or none.

        Returns true, false or none.
        '''
        return self._show_identity_operators

    ### PUBLIC METHODS ###

    def duplicate(self, counts=None, indices=None, period=None):
        r'''Configures compound operator to duplicate pitches by `counts`, with
        optional `indices` and `period`.

        ..  container:: example

            ::

                >>> operator = pitchtools.CompoundOperator()
                >>> operator = operator.duplicate(counts=1)
                >>> f(operator)
                pitchtools.CompoundOperator(
                    operators=[
                        sequencetools.Duplication(
                            counts=1,
                            ),
                        ],
                    )

        Returns new compound operator.
        '''
        operator = sequencetools.Duplication(
            counts=counts,
            indices=indices,
            period=period,
            )
        return self._with_operator(operator)

    def invert(self, axis=None):
        r'''Configures compound operator to invert pitches about `axis`.

        ..  container:: example

            ::

                >>> operator = pitchtools.CompoundOperator()
                >>> operator = operator.invert(axis=2)
                >>> f(operator)
                pitchtools.CompoundOperator(
                    operators=[
                        pitchtools.Inversion(
                            axis=pitchtools.NamedPitch("d'"),
                            ),
                        ],
                    )

        Returns new compound operator.
        '''
        from abjad.tools import pitchtools
        operator = pitchtools.Inversion(axis=axis)
        return self._with_operator(operator)

    def multiply(self, n=1):
        r'''Configures compound operator to multiply pitch-classes by index `n`.

        ..  container:: example

            ::

                >>> operator = pitchtools.CompoundOperator()
                >>> operator = operator.multiply(n=3)
                >>> f(operator)
                pitchtools.CompoundOperator(
                    operators=[
                        pitchtools.Multiplication(
                            n=3,
                            ),
                        ],
                    )

        Returns new compound operator.
        '''
        from abjad.tools import pitchtools
        operator = pitchtools.Multiplication(n=n)
        return self._with_operator(operator)

    def retrograde(self, period=None):
        r'''Configures compound operator to retrograde pitches.

        ..  container:: example

            ::

                >>> operator = pitchtools.CompoundOperator()
                >>> operator = operator.retrograde()
                >>> f(operator)
                pitchtools.CompoundOperator(
                    operators=[
                        pitchtools.Retrograde(),
                        ],
                    )

        Returns new compound operator.
        '''
        from abjad.tools import pitchtools
        operator = pitchtools.Retrograde(period=period)
        return self._with_operator(operator)

    def rotate(self, n=0, period=None, stravinsky=None):
        r'''Configures compound operator to rotate pitches by index `n`.

        ..  container:: example

            ::

                >>> operator = pitchtools.CompoundOperator()
                >>> operator = operator.rotate(n=-1)
                >>> f(operator)
                pitchtools.CompoundOperator(
                    operators=[
                        pitchtools.Rotation(
                            n=-1,
                            ),
                        ],
                    )

        Returns new compound operator.
        '''
        from abjad.tools import pitchtools
        operator = pitchtools.Rotation(
            n=n,
            period=period,
            stravinsky=stravinsky,
            )
        return self._with_operator(operator)

    def transpose(self, n=0):
        r'''Configures compound operator to transpose pitches by index `n`.

        ..  container:: example

            ::

                >>> operator = pitchtools.CompoundOperator()
                >>> operator = operator.transpose(n=1)

            ::

                >>> f(operator)
                pitchtools.CompoundOperator(
                    operators=[
                        pitchtools.Transposition(
                            n=1,
                            ),
                        ],
                    )

        Returns new compound operator.
        '''
        from abjad.tools import pitchtools
        operator = pitchtools.Transposition(n=n)
        return self._with_operator(operator)



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedPitch_pitch_class_number_01():

    assert NamedPitch('c').pitch_class_number == 0
    assert NamedPitch('d').pitch_class_number == 2
    assert NamedPitch('e').pitch_class_number == 4
    assert NamedPitch('f').pitch_class_number == 5
    assert NamedPitch('g').pitch_class_number == 7
    assert NamedPitch('a').pitch_class_number == 9
    assert NamedPitch('b').pitch_class_number == 11


def test_pitchtools_NamedPitch_pitch_class_number_02():

    assert NamedPitch('cff').pitch_class_number == 10
    assert NamedPitch('ctqf').pitch_class_number == 10.5
    assert NamedPitch('cf').pitch_class_number == 11
    assert NamedPitch('cqf').pitch_class_number == 11.5
    assert NamedPitch('c').pitch_class_number == 0
    assert NamedPitch('cqs').pitch_class_number == 0.5
    assert NamedPitch('cs').pitch_class_number == 1
    assert NamedPitch('ctqs').pitch_class_number == 1.5
    assert NamedPitch('css').pitch_class_number == 2


def test_pitchtools_NamedPitch_pitch_class_number_03():

    assert NamedPitch("cf''").pitch_class_number == 11
    assert NamedPitch("c''").pitch_class_number == 0
    assert NamedPitch("cs''").pitch_class_number == 1
    assert NamedPitch("bf''").pitch_class_number == 10
    assert NamedPitch("b''").pitch_class_number == 11
    assert NamedPitch("bs''").pitch_class_number == 0



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchClassSegment_invert_01():

    pitch_class_segment = pitchtools.PitchClassSegment([0, 6, 10, 4, 9, 2])

    assert pitch_class_segment.invert() == \
        pitchtools.PitchClassSegment([0, 6, 2, 8, 3, 10])



# -*- coding: utf-8 -*-
from abjad.tools.abctools import AbjadObject


class UpdateManager(AbjadObject):
    '''Update manager.

    Updates start offset, stop offsets and indicators everywhere in score.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Managers'

    ### PRIVATE METHODS ###

    @staticmethod
    def _get_score_tree_state_flags(parentage):
        offsets_are_current = True
        indicators_are_current = True
        offsets_in_seconds_are_current = True
        for component in parentage:
            if offsets_are_current:
                if not component._offsets_are_current:
                    offsets_are_current = False
            if indicators_are_current:
                if not component._indicators_are_current:
                    indicators_are_current = False
            if offsets_in_seconds_are_current:
                if not component._offsets_in_seconds_are_current:
                    offsets_in_seconds_are_current = False
        return (
            offsets_are_current,
            indicators_are_current,
            offsets_in_seconds_are_current,
            )

    @staticmethod
    def _iterate_entire_score(score_root):
        from abjad.tools.topleveltools import iterate
        components = iterate(score_root).depth_first(
            capped=True,
            unique=True,
            forbid=None,
            direction=Left,
            )
        return components

    def _update_all_indicators(self, score_root):
        r'''Updating indicators does not update offsets.
        On the other hand, getting an effective indicator does update
        offsets when at least one indicator of the appropriate type
        attaches to score.
        '''
        components = self._iterate_entire_score(score_root)
        for component in components:
            for indicator in component._get_indicators(unwrap=False):
                if indicator.scope is not None:
                    assert hasattr(indicator, '_update_effective_context')
                    indicator._update_effective_context()
            component._indicators_are_current = True

    @staticmethod
    def _update_all_leaf_indices_and_measure_numbers(score_root):
        r'''Call only when updating offsets.
        No separate state flags exist for leaf indices or measure numbers.
        '''
        from abjad.tools import scoretools
        from abjad.tools.topleveltools import iterate
        if isinstance(score_root, scoretools.Context):
            contexts = iterate(score_root).by_class(scoretools.Context)
            for context in contexts:
                for leaf_index, leaf in enumerate(iterate(context).by_leaf()):
                    leaf._leaf_index = leaf_index
                for measure_index, measure in enumerate(
                    iterate(context).by_class(scoretools.Measure)):
                    measure_number = measure_index + 1
                    measure._measure_number = measure_number
        else:
            for leaf_index, leaf in enumerate(iterate(score_root).by_leaf()):
                leaf._leaf_index = leaf_index
            for measure_index, measure in enumerate(
                iterate(score_root).by_class(scoretools.Measure)):
                measure_number = measure_index + 1
                measure._measure_number = measure_number

    def _update_all_offsets(self, score_root):
        r'''Updating offsets does not update indicators.
        Updating offsets does not update offsets in seconds.
        '''
        for component in self._iterate_entire_score(score_root):
            self._update_component_offsets(component)
            component._offsets_are_current = True

    def _update_all_offsets_in_seconds(self, score_root):
        for component in self._iterate_entire_score(score_root):
            self._update_component_offsets_in_seconds(component)
            component._offsets_in_seconds_are_current = True

    @classmethod
    def _update_component_offsets(class_, component):
        from abjad.tools import durationtools
        from abjad.tools import scoretools
        if (isinstance(component._parent, scoretools.GraceContainer) and
            not component._parent.kind == 'after'):
            pair = class_._get_grace_note_offsets(component)
            start_offset, stop_offset = pair
        elif (isinstance(component._parent, scoretools.GraceContainer) and
            component._parent.kind == 'after'):
            pair = class_._get_after_grace_note_offsets(component)
            start_offset, stop_offset = pair
        else:
            previous = component._get_nth_component_in_time_order_from(-1)
            if previous is not None:
                start_offset = previous._stop_offset
            else:
                start_offset = durationtools.Offset(0)
            stop_offset = start_offset + component._get_duration()
        component._start_offset = start_offset
        component._stop_offset = stop_offset
        component._timespan._start_offset = start_offset
        component._timespan._stop_offset = stop_offset

    @staticmethod
    def _update_component_offsets_in_seconds(component):
        from abjad.tools import durationtools
        try:
            current_duration_in_seconds = \
                component._get_duration(in_seconds=True)
            previous = component._get_nth_component_in_time_order_from(-1)
            if previous is not None:
                component._start_offset_in_seconds = \
                    previous._get_timespan(in_seconds=True)._stop_offset
            else:
                component._start_offset_in_seconds = durationtools.Offset(0)
            # this one case is possible for containers only
            if component._start_offset_in_seconds is None:
                raise MissingTempoError
            component._stop_offset_in_seconds = \
                component._start_offset_in_seconds + \
                current_duration_in_seconds
        except MissingTempoError:
            pass

    def _update_now(
        self,
        component,
        offsets=False,
        offsets_in_seconds=False,
        indicators=False,
        ):
        assert offsets or offsets_in_seconds or indicators
        if component._is_forbidden_to_update:
            return
        parentage = component._get_parentage(
            include_self=True,
            with_grace_notes=True,
            )
        for parent in parentage:
            if parent._is_forbidden_to_update:
                return
        (
            offsets_are_current,
            indicators_are_current,
            offsets_in_seconds_are_current,
            ) = self._get_score_tree_state_flags(parentage)
        score_root = parentage.root
        if offsets and not offsets_are_current:
            self._update_all_offsets(score_root)
            self._update_all_leaf_indices_and_measure_numbers(score_root)
        if offsets_in_seconds and not offsets_in_seconds_are_current:
            self._update_all_offsets_in_seconds(score_root)
        if indicators and not indicators_are_current:
            self._update_all_indicators(score_root)
            self._update_all_offsets_in_seconds(score_root)

    ### EXPERIMENTAL ###

    @staticmethod
    def _get_after_grace_note_offsets(grace_note):
        from abjad.tools import durationtools
        after_grace_container = grace_note._parent
        assert after_grace_container.kind == 'after'
        carrier_leaf = after_grace_container._carrier
        carrier_leaf_stop_offset = carrier_leaf._stop_offset
        grace_displacement = -grace_note.written_duration
        sibling = grace_note._get_sibling(1)
        while sibling is not None:
            grace_displacement -= sibling.written_duration
            sibling = sibling._get_sibling(1)
        start_offset = durationtools.Offset(
            carrier_leaf_stop_offset,
            grace_displacement=grace_displacement,
            )
        grace_displacement += grace_note.written_duration
        stop_offset = durationtools.Offset(
            carrier_leaf_stop_offset,
            grace_displacement=grace_displacement,
            )
        return start_offset, stop_offset

    @staticmethod
    def _get_grace_note_offsets(grace_note):
        from abjad.tools import durationtools
        grace_container = grace_note._parent
        carrier_leaf = grace_container._carrier
        carrier_leaf_start_offset = carrier_leaf._start_offset
        grace_displacement = -grace_note.written_duration
        sibling = grace_note._get_sibling(1)
        while sibling is not None:
            grace_displacement -= sibling.written_duration
            sibling = sibling._get_sibling(1)
        start_offset = durationtools.Offset(
            carrier_leaf_start_offset,
            grace_displacement=grace_displacement,
            )
        grace_displacement += grace_note.written_duration
        stop_offset = durationtools.Offset(
            carrier_leaf_start_offset,
            grace_displacement=grace_displacement,
            )
        return start_offset, stop_offset

    def _get_logical_measure_start_offsets(self, component):
        from abjad.tools import durationtools
        from abjad.tools import indicatortools
        from abjad.tools import sequencetools
        from abjad.tools.topleveltools import inspect_
        expressions = []
        prototype = indicatortools.TimeSignature
        score_root = component._get_parentage(include_self=True).root
        for component in self._iterate_entire_score(score_root):
            expressions_ = component._get_indicators(
                prototype,
                unwrap=False,
                )
            expressions.extend(expressions_)
        pairs = []
        for expression in expressions:
            inspector = inspect_(expression.component)
            start_offset = inspector.get_timespan()._start_offset
            time_signature = expression.indicator
            pair = start_offset, time_signature
            pairs.append(pair)
        offset_zero = durationtools.Offset(0)
        default_time_signature = indicatortools.TimeSignature((4, 4))
        default_pair = (offset_zero, default_time_signature)
        if pairs and not pairs[0] == offset_zero:
            pairs.insert(0, default_pair)
        elif not pairs:
            pairs = [default_pair]
        pairs.sort(key=lambda x: x[0])
        parentage = component._get_parentage()
        score_root = parentage.root
        inspector = inspect_(score_root)
        score_stop_offset = inspector.get_timespan()._stop_offset
        dummy_last_pair = (score_stop_offset, None)
        pairs.append(dummy_last_pair)
        measure_start_offsets = []
        for current_pair, next_pair in sequencetools.Sequence(pairs).nwise():
            current_start_offset, current_time_signature = current_pair
            next_start_offset, next_time_signature = next_pair
            measure_start_offset = current_start_offset
            while measure_start_offset < next_start_offset:
                measure_start_offsets.append(measure_start_offset)
                measure_start_offset += current_time_signature.duration
        return measure_start_offsets

    # TODO: reimplement with some type of bisection
    def _to_logical_measure_number(
        self,
        component,
        logical_measure_number_start_offsets,
        ):
        from abjad.tools import mathtools
        from abjad.tools import sequencetools
        from abjad.tools.topleveltools import inspect_
        inspector = inspect_(component)
        component_start_offset = inspector.get_timespan()._start_offset
        logical_measure_number_start_offsets = \
            logical_measure_number_start_offsets[:]
        logical_measure_number_start_offsets.append(mathtools.Infinity())
        pairs = sequencetools.Sequence(logical_measure_number_start_offsets)
        pairs = pairs.nwise()
        for logical_measure_index, pair in enumerate(pairs):
            if pair[0] <= component_start_offset < pair[-1]:
                logical_measure_number = logical_measure_index + 1
                return logical_measure_number
        message = 'can not find logical measure number: {!r}, {!r}.'
        message = message.format(
            component,
            logical_measure_number_start_offsets,
            )
        raise ValueError(message)

    def _update_logical_measure_numbers(self, component):
        logical_measure_start_offsets = \
            self._get_logical_measure_start_offsets(component)
        assert logical_measure_start_offsets, repr(
            logical_measure_start_offsets)
        score_root = component._get_parentage(include_self=True).root
        for component in self._iterate_entire_score(score_root):
            logical_measure_number = self._to_logical_measure_number(
                component,
                logical_measure_start_offsets,
                )
            component._logical_measure_number = logical_measure_number



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_TrillSpanner___init___01():
    r'''Initialize empty trill spanner.
    '''

    trill = spannertools.TrillSpanner()
    assert isinstance(trill, spannertools.TrillSpanner)



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Measure_duration_01():
    r'''Properly filled measure with power-of-two time signature.
    '''

    measure = Measure((3, 8), "c'8 d'8 e'8")

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 3/8
            c'8
            d'8
            e'8
        }
        '''
        )

    assert measure._contents_duration == Duration(3, 8)
    assert measure._preprolated_duration == Duration(3, 8)
    assert inspect_(measure).get_duration() == Duration(3, 8)


def test_scoretools_Measure_duration_02():
    r'''Properly filled measure with non-power-of-two time signature.
    '''

    measure = Measure((3, 10), "c'8 d'8 e'8")
    measure.implicit_scaling = True

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 3/10
            \scaleDurations #'(4 . 5) {
                c'8
                d'8
                e'8
            }
        }
        '''
        )

    assert measure._contents_duration == Duration(3, 8)
    assert measure._preprolated_duration == Duration(3, 10)
    assert inspect_(measure).get_duration() == Duration(3, 10)


def test_scoretools_Measure_duration_03():
    r'''Improperly filled measure with power-of-two time signature.
    '''

    measure = Measure((3, 8), "c'8 d'8 e'8 f'8")

    assert pytest.raises(OverfullContainerError, 'format(measure)')

    assert measure._contents_duration == Duration(4, 8)
    assert measure._preprolated_duration == Duration(4, 8)
    assert inspect_(measure).get_duration() == Duration(4, 8)


def test_scoretools_Measure_duration_04():
    r'''Impropely filled measure with non-power-of-two time signature.
    '''

    measure = Measure((3, 10), "c'8 d'8 e'8 f'8")
    measure.implicit_scaling = True

    assert pytest.raises(OverfullContainerError, 'format(measure)')

    assert measure._contents_duration == Duration(4, 8)
    assert measure._preprolated_duration == Duration(4, 10)
    assert inspect_(measure).get_duration() == Duration(4, 10)



# -*- coding: utf-8 -*-
from abjad import *


def test_datastructuretools_TreeContainer___iter___01():

    leaf_a = datastructuretools.TreeNode()
    leaf_b = datastructuretools.TreeNode()
    leaf_c = datastructuretools.TreeNode()

    container = datastructuretools.TreeContainer([leaf_a, leaf_b, leaf_c])

    assert [x for x in container] == [leaf_a, leaf_b, leaf_c]



# -*- coding: utf-8 -*-
from abjad import *


def test_abctools_AbjadObject___repr___01():

    class Foo(abctools.AbjadObject):
        def __init__(self, x, y, flavor=None):
            tmp_1 = 'foo'
            self.x = x
            self.y = y
            self.flavor = flavor
            tmp_2 = 'bar'

    assert repr(Foo(7, 8)) == 'Foo(7, 8)'
    assert repr(Foo(7, 8, 'cherry')) == "Foo(7, 8, flavor='cherry')"


def test_abctools_AbjadObject___repr___02():
    r'''Repr suppresses class methods to avoid recursive repr.
    '''

    class Foo(abctools.AbjadObject):
        def __init__(self, x, y, helper=None):
            tmp_1 = 'foo'
            self.x = x
            self.y = y
            self.helper = helper or self.do_stuff
            tmp_2 = 'bar'
        def do_stuff(self):
            pass

    assert repr(Foo(7, 8)) == 'Foo(7, 8)'



# -*- coding: utf-8 -*-
import copy
from abjad import *


def test_lilypondproxytools_LilyPondSettingNameManager___eq___01():

    note_1 = Note("c'4")
    set_(note_1).voice.auto_beaming = False
    set_(note_1).staff.tuplet_full_length = True

    note_2 = Note("c'4")
    set_(note_2).voice.auto_beaming = False
    set_(note_2).staff.tuplet_full_length = True

    note_3 = Note("c'4")
    set_(note_3).voice.auto_beaming = True

    context_setting_component_plug_in_1 = set_(note_1)
    context_setting_component_plug_in_2 = set_(note_2)
    context_setting_component_plug_in_3 = set_(note_3)

    assert      context_setting_component_plug_in_1 == context_setting_component_plug_in_1
    assert      context_setting_component_plug_in_1 == context_setting_component_plug_in_2
    assert not context_setting_component_plug_in_1 == context_setting_component_plug_in_3
    assert      context_setting_component_plug_in_2 == context_setting_component_plug_in_1
    assert      context_setting_component_plug_in_2 == context_setting_component_plug_in_2
    assert not context_setting_component_plug_in_2 == context_setting_component_plug_in_3
    assert not context_setting_component_plug_in_3 == context_setting_component_plug_in_1
    assert not context_setting_component_plug_in_3 == context_setting_component_plug_in_2
    assert      context_setting_component_plug_in_3 == context_setting_component_plug_in_3



# -*- coding: utf-8 -*-
from abjad.tools.abctools import AbjadObject


class ChordInversion(AbjadObject):
    '''A chord inversion for tertian chords: 5, 63, 64 and
    also 7, 65, 43, 42, etc.

    Also root position, first, second, third
    inversions, etc.

    Value object that can not be changed once initialized.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_number',
        )

    _inversion_name_to_inversion_number = {
        'root': 0,
        'root position': 0,
        'first': 1,
        'second': 2,
        'third': 3,
        'fourth': 4,
        }

    _inversion_number_to_inversion_name = {
        0: 'root position',
        1: 'first',
        2: 'second',
        3: 'third',
        4: 'fourth',
        }

    _seventh_chord_inversion_to_figured_bass_string = {
        0: '7',
        1: '6/5',
        2: '4/3',
        3: '4/2',
        }

    _triadic_inversion_to_figured_bass_string = {
        0: '',
        1: '6',
        2: '6/4',
        }

    ### INITIALIZER ###

    def __init__(self, number=0):
        arg = number
        if isinstance(arg, int):
            number = arg
        elif isinstance(arg, str):
            number = self._inversion_name_to_inversion_number[arg]
        else:
            message = 'can not initialize chord inversion.'
            raise ValueError(message)
        self._number = number

    ### SPECIAL METHODS ###

    def __eq__(self, arg):
        r'''Is true when `arg` is a chord inversion with number equal to that
        of this chord inversion. Otherwise false.

        Returns true or false.
        '''
        if isinstance(arg, type(self)):
            if self.number == arg.number:
                return True
        return False

    def __hash__(self):
        r'''Hashes chord inversion.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(ChordInversion, self).__hash__()

    def __ne__(self, arg):
        r'''Is true when chord inversion does not equal `arg`. Otherise false.

        Returns true or false.
        '''
        return not self == arg

    ### PUBLIC METHODS ###

    def extent_to_figured_bass_string(self, extent):
        r'''Changes `extent` to figured bass string.

        Returns string.
        '''
        if extent == 5:
            return self._triadic_inversion_to_figured_bass_string[self.number]
        elif extent == 7:
            return self._seventh_chord_inversion_to_figured_bass_string[
                self.number]
        else:
            raise NotImplementedError

    ### PUBLIC PROPERTIES ###

    @property
    def name(self):
        r'''Name of chord inversion.

        Returns string.
        '''
        return self._inversion_number_to_inversion_name[self.number]

    @property
    def number(self):
        r'''Number of chord inversion.

        Returns nonnegative integer.
        '''
        return self._number

    @property
    def title(self):
        r'''Title of chord inversion.

        Returns string.
        '''
        name = self._inversion_number_to_inversion_name[self.number]
        if name == 'root position':
            return 'RootPosition'
        return '{}Inversion'.format(name.title())



# -*- coding: utf-8 -*-
from abjad import *


def test_instrumenttools_transpose_from_sounding_pitch_to_written_pitch_01():

    staff = Staff("<c' e' g'>4 d'4 r4 e'4")
    clarinet = instrumenttools.ClarinetInBFlat()
    attach(clarinet, staff)
    instrumenttools.transpose_from_sounding_pitch_to_written_pitch(staff)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \set Staff.instrumentName = \markup { "Clarinet in B-flat" }
            \set Staff.shortInstrumentName = \markup { "Cl. in B-flat" }
            <d' fs' a'>4
            e'4
            r4
            fs'4
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchRangeList___contains___01():

    pitch_ranges = pitchtools.PitchRangeList(['[C3, C6]', '[C4, C6]'])

    assert '[C3, C6]' in pitch_ranges
    assert (-12, 24) in pitch_ranges
    assert (-39, 48) not in pitch_ranges



# -*- coding: utf-8 -*-
from abjad.tools.durationtools import Offset
from abjad.tools.rhythmtreetools import RhythmTreeContainer, RhythmTreeLeaf


def test_rhythmtreetools_RhythmTreeNode_offset_01():

    tree = RhythmTreeContainer(preprolated_duration=1, children=[
        RhythmTreeLeaf(preprolated_duration=1),
        RhythmTreeContainer(preprolated_duration=2, children=[
            RhythmTreeLeaf(preprolated_duration=3),
            RhythmTreeLeaf(preprolated_duration=2)
        ]),
        RhythmTreeLeaf(preprolated_duration=2)
    ])

    assert tree.start_offset == Offset(0)
    assert tree[0].start_offset == Offset(0)
    assert tree[1].start_offset == Offset(1, 5)
    assert tree[1][0].start_offset == Offset(1, 5)
    assert tree[1][1].start_offset == Offset(11, 25)
    assert tree[2].start_offset == Offset(3, 5)

    node = tree.pop()

    assert node.start_offset == Offset(0)

    tree[1].append(node)

    assert tree.start_offset == Offset(0)
    assert tree[0].start_offset == Offset(0)
    assert tree[1].start_offset == Offset(1, 3)
    assert tree[1][0].start_offset == Offset(1, 3)
    assert tree[1][1].start_offset == Offset(13, 21)
    assert tree[1][2].start_offset == Offset(17, 21)

    tree.preprolated_duration = 19

    assert tree.start_offset == Offset(0)
    assert tree[0].start_offset == Offset(0)
    assert tree[1].start_offset == Offset(19, 3)
    assert tree[1][0].start_offset == Offset(19, 3)
    assert tree[1][1].start_offset == Offset(247, 21)
    assert tree[1][2].start_offset == Offset(323, 21)



# -*- coding: utf-8 -*-
import collections
from abjad.tools import sequencetools
from abjad.tools.pitchtools.Set import Set
from abjad.tools.topleveltools import new


class PitchSet(Set):
    r'''Pitch set.

    ..  container:: example

        Numbered pitch set:

        ::

            >>> set_ = PitchSet(
            ...     items=[-2, -1.5, 6, 7, -1.5, 7],
            ...     item_class=NumberedPitch,
            ...     )
            >>> set_
            PitchSet([-2, -1.5, 6, 7])

        ::

            >>> f(set_)
            pitchtools.PitchSet(
                [-2, -1.5, 6, 7]
                )

    ..  container:: example

        Named pitch set:

        ::

            >>> set_ = PitchSet(
            ...     ['bf,', 'aqs', "fs'", "g'", 'bqf', "g'"],
            ...     item_class=NamedPitch,
            ...     )
            >>> set_
            PitchSet(['bf,', 'aqs', 'bqf', "fs'", "g'"])

        ::

            >>> f(set_)
            pitchtools.PitchSet(
                ['bf,', 'aqs', 'bqf', "fs'", "g'"]
                )

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### SPECIAL METHODS ###

    def __illustrate__(self):
        r'''Illustrates pitch set.

        ..  container:: example

            Treble and bass pitches:

            ::

                >>> set_ = PitchSet(
                ...     items=[-2, -1.5, 6, 7, -1.5, 7],
                ...     item_class=NumberedPitch,
                ...     )

            ::
            
                >>> show(set_) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = set_.__illustrate__()
                >>> f(lilypond_file[Score])
                \new Score <<
                    \new PianoStaff <<
                        \new Staff {
                            \new Voice {
                                <fs' g'>1
                            }
                        }
                        \new Staff {
                            \new Voice {
                                <bf bqf>1
                            }
                        }
                    >>
                >>

        ..  container:: example

            Treble pitches only:

            ::

                >>> set_ = PitchSet(
                ...     items=[6, 7, 7],
                ...     item_class=NumberedPitch,
                ...     )

            ::
            
                >>> show(set_) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = set_.__illustrate__()
                >>> f(lilypond_file[Score])
                \new Score <<
                    \new PianoStaff <<
                        \new Staff {
                            \new Voice {
                                <fs' g'>1
                            }
                        }
                        \new Staff {
                            \new Voice {
                                s1
                            }
                        }
                    >>
                >>

        Returns LilyPond file.
        '''
        import abjad
        upper, lower = [], []
        for pitch in self:
            if pitch < 0:
                lower.append(pitch)
            else:
                upper.append(pitch)
        if upper:
            upper = abjad.Chord(upper, abjad.Duration(1))
        else:
            upper = abjad.Skip((1, 1))
        if lower:
            lower = abjad.Chord(lower, abjad.Duration(1))
        else:
            lower = abjad.Skip((1, 1))
        upper_voice = abjad.Voice([upper])
        upper_staff = abjad.Staff([upper_voice])
        lower_voice = abjad.Voice([lower])
        lower_staff = abjad.Staff([lower_voice])
        staff_group = abjad.StaffGroup(
            [upper_staff, lower_staff],
            context_name='PianoStaff',
            )
        score = abjad.Score([staff_group])
        lilypond_file = abjad.LilyPondFile.new(score)
        lilypond_file.header_block.tagline = False
        return lilypond_file

    ### PRIVATE PROPERTIES ###

    @property
    def _named_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NamedPitch

    @property
    def _numbered_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NumberedPitch

    @property
    def _parent_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.Pitch

    ### PRIVATE METHODS ###

    def _is_equivalent_under_transposition(self, argument):
        r'''True if pitch set is equivalent to `argument` under transposition.
        Otherwise false.

        Returns true or false.
        '''
        from abjad.tools import pitchtools
        if not isinstance(argument, type(self)):
            return False
        if not len(self) == len(argument):
            return False
        difference = -(pitchtools.NamedPitch(argument[0], 4) -
            pitchtools.NamedPitch(self[0], 4))
        new_pitches = (x + difference for x in self)
        new_pitches = new(self, items=new_pitch)
        return argument == new_pitches

    def _sort_self(self):
        from abjad.tools import pitchtools
        return sorted(pitchtools.PitchSegment(tuple(self)))

    ### PUBLIC PROPERTIES ###

    @property
    def duplicate_pitch_classes(self):
        r'''Gets duplicate pitch-classes in pitch set.

        Returns pitch-class set.
        '''
        from abjad.tools import pitchtools
        pitch_classes = []
        duplicate_pitch_classes = []
        for pitch in self:
            pitch_class = pitchtools.NumberedPitchClass(pitch)
            if pitch_class in pitch_classes:
                duplicate_pitch_classes.append(pitch_class)
            pitch_classes.append(pitch_class)
        return pitchtools.PitchClassSet(
            duplicate_pitch_classes,
            item_class=pitchtools.NumberedPitchClass,
            )

    @property
    def hertz(self):
        r'''Gets hertz of pitches in pitch segment.

        ..  container:: example

            ::

                >>> pitch_set = PitchSet('c e g b')
                >>> sorted(pitch_set.hertz)
                [130.81..., 164.81..., 195.99..., 246.94...]

        Returns set.
        '''
        return set(_.hertz for _ in self)

    @property
    def is_pitch_class_unique(self):
        r'''Is true when pitch set is pitch-class-unique. Otherwise false.

        Returns true or false.
        '''
        from abjad.tools import pitchtools
        numbered_pitch_class_set = pitchtools.PitchClassSet(
            self, item_class=pitchtools.NumberedPitchClass)
        return len(self) == len(numbered_pitch_class_set)

    ### PUBLIC METHODS ###

    @classmethod
    def from_selection(
        class_,
        selection,
        item_class=None,
        ):
        r'''Makes pitch set from `selection`.

        ..  container:: example

            ::

                >>> staff_1 = Staff("c'4 <d' fs' a'>4 b2")
                >>> staff_2 = Staff("c4. r8 g2")
                >>> selection = select((staff_1, staff_2))
                >>> PitchSet.from_selection(selection)
                PitchSet(['c', 'g', 'b', "c'", "d'", "fs'", "a'"])

        Returns pitch set.
        '''
        from abjad.tools import pitchtools
        pitch_segment = pitchtools.PitchSegment.from_selection(selection)
        return class_(
            items=pitch_segment,
            item_class=item_class,
            )

    def invert(self, axis):
        r'''Inverts pitch set about `axis`.

        Returns new pitch set.
        '''
        items = (pitch.invert(axis) for pitch in self)
        return new(self, items=items)

    def register(self, pitch_classes):
        '''Registers `pitch_classes` by pitch set.

        ..  container:: example

            ::

                >>> pitch_set = PitchSet(
                ...     items=[10, 19, 20, 23, 24, 26, 27, 29, 30, 33, 37, 40],
                ...     item_class=NumberedPitch,
                ...     )
                >>> pitch_classes = [10, 0, 2, 6, 8, 7, 5, 3, 1, 9, 4, 11]
                >>> pitches = pitch_set.register(pitch_classes)
                >>> for pitch in pitches:
                ...     pitch
                NumberedPitch(10)
                NumberedPitch(24)
                NumberedPitch(26)
                NumberedPitch(30)
                NumberedPitch(20)
                NumberedPitch(19)
                NumberedPitch(29)
                NumberedPitch(27)
                NumberedPitch(37)
                NumberedPitch(33)
                NumberedPitch(40)
                NumberedPitch(23)

        Returns list of zero or more numbered pitches.
        '''
        if isinstance(pitch_classes, collections.Iterable):
            result = [
                [_ for _ in self if _.pitch_number % 12 == pc]
                for pc in [x % 12 for x in pitch_classes]
                ]
            result = sequencetools.Sequence(result).flatten()
        elif isinstance(pitch_classes, int):
            result = [p for p in pitch_classes if p % 12 == pitch_classes][0]
        else:
            message = 'must be pitch-class or list of pitch-classes.'
            raise TypeError(message)
        return result

    def transpose(self, n=0):
        r'''Transposes pitch set by index `n`.

        Returns new pitch set.
        '''
        items = (pitch.transpose(n=n) for pitch in self)
        return new(self, items=items)



# -*- coding: utf-8 -*-


def sequence(items=None, **keywords):
    r'''Makes sequence or sequence expression.

    ..  container:: example

        ..  container:: example

            Makes sequence:

            ::

                >>> sequence([1, 2, [3, [4]], 5])
                Sequence([1, 2, [3, [4]], 5])

        ..  container:: example expression

            Makes sequence expression:

            ::

                >>> expression = sequence()
                >>> expression([1, 2, [3, [4]], 5])
                Sequence([1, 2, [3, [4]], 5])

    ..  container:: example

        Flattens, reverses and slices sequence:

        ..  container:: example

            ::

                >>> sequence_ = sequence([1, 2, [3, [4]], 5])
                >>> sequence_
                Sequence([1, 2, [3, [4]], 5])

            ::

                >>> sequence_ = sequence_.flatten()
                >>> sequence_
                Sequence([1, 2, 3, 4, 5])

            ::

                >>> sequence_ = sequence_.reverse()
                >>> sequence_
                Sequence([5, 4, 3, 2, 1])

            ::

                >>> sequence_ = sequence_[-3:]
                >>> sequence_
                Sequence([3, 2, 1])

        ..  container:: example expression

            ::

                >>> expression = sequence()
                >>> expression = sequence()
                >>> expression = expression.flatten()
                >>> expression = expression.reverse()
                >>> expression = expression[-3:]
                >>> expression([1, 2, [3, [4]], 5])
                Sequence([3, 2, 1])
        
    Returns sequence when `items` is not none.

    Returns sequence expression when `items` is none.
    '''
    from abjad.tools import expressiontools
    from abjad.tools import sequencetools
    if items:
        return sequencetools.Sequence(items=items, **keywords)
    else:
        expression = expressiontools.Expression()
        expression = expression.sequence(**keywords)
        return expression



# -*- coding: utf-8 -*-
from abjad.tools import mathtools
from abjad.tools.quantizationtools.SearchTree import SearchTree


class UnweightedSearchTree(SearchTree):
    r'''Concrete ``SearchTree`` subclass, based on Paul Nauert's search
    tree model:

    ::

        >>> search_tree = quantizationtools.UnweightedSearchTree()
        >>> print(format(search_tree))
        quantizationtools.UnweightedSearchTree(
            definition={
                2: {
                    2: {
                        2: {
                            2: None,
                            },
                        3: None,
                        },
                    3: None,
                    5: None,
                    7: None,
                    },
                3: {
                    2: {
                        2: None,
                        },
                    3: None,
                    5: None,
                    },
                5: {
                    2: None,
                    3: None,
                    },
                7: {
                    2: None,
                    },
                11: None,
                13: None,
                },
            )

    The search tree defines how nodes in a ``QGrid`` may be subdivided,
    if they happen to contain ``QEvents`` (or, in actuality, ``QEventProxy``
    instances which reference ``QEvents``, but rescale their offsets between
    ``0`` and ``1``).

    In the default definition, the root node of the ``QGrid`` may be
    subdivided into ``2``, ``3``, ``5``, ``7``, ``11`` or ``13`` equal parts.
    If divided into ``2`` parts, the divisions of the root node may be
    divided again into ``2``, ``3``, ``5`` or ``7``, and so forth.

    This definition is structured as a collection of nested dictionaries,
    whose keys are integers, and whose values are either the sentinel ``None``
    indicating no further permissable divisions, or dictionaries obeying
    these same rules, which then indicate the possibilities for further
    division.

    Calling a ``UnweightedSearchTree`` with a ``QGrid`` instance will
    generate all permissable subdivided ``QGrids``, according to the
    definition of the called search tree:

    ::

        >>> q_event_a = quantizationtools.PitchedQEvent(130, [0, 1, 4])
        >>> q_event_b = quantizationtools.PitchedQEvent(150, [2, 3, 5])
        >>> proxy_a = quantizationtools.QEventProxy(q_event_a, 0.5)
        >>> proxy_b = quantizationtools.QEventProxy(q_event_b, 0.667)
        >>> q_grid = quantizationtools.QGrid()
        >>> q_grid.fit_q_events([proxy_a, proxy_b])

    ::

        >>> q_grids = search_tree(q_grid)
        >>> for grid in q_grids:
        ...     print(grid.rtm_format)
        (1 (1 1))
        (1 (1 1 1))
        (1 (1 1 1 1 1))
        (1 (1 1 1 1 1 1 1))
        (1 (1 1 1 1 1 1 1 1 1 1 1))
        (1 (1 1 1 1 1 1 1 1 1 1 1 1 1))

    A custom ``UnweightedSearchTree`` may be defined by passing in a
    dictionary, as described above.
    The following search tree only permits divisions of
    the root node into ``2s`` and ``3s``, and if divided into ``2``,
    a node may be divided once more into ``2`` parts:

    ::

        >>> definition = {2: {2: None}, 3: None}
        >>> search_tree = quantizationtools.UnweightedSearchTree(definition)

    ::

        >>> q_grids = search_tree(q_grid)
        >>> for grid in q_grids:
        ...     print(grid.rtm_format)
        (1 (1 1))
        (1 (1 1 1))

    Return ``UnweightedSearchTree`` instance.
    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### PRIVATE METHODS ###

    def _find_leaf_subdivisions(self, parentage_ratios):
        parentage = [x[1] for x in parentage_ratios[1:]]
        if not parentage:
            return tuple((1,) * x for x in sorted(self._definition.keys()))
        node = self._definition[parentage[0]]
        for item in parentage[1:]:
            node = node[item]
            if node is None:
                return ()
        if node is None:
            return ()
        return tuple((1,) * x for x in sorted(node.keys()))

    def _is_valid_definition(self, definition):
        def recurse(n):
            results = []
            for key in n:
                if not isinstance(key, int) or \
                    not 0 < key or \
                    not mathtools.divisors(key) == [1, key]:
                    results.append(False)
                elif not isinstance(n[key], (dict, type(None))):
                    results.append(False)
                elif isinstance(n[key], dict) and not recurse(n[key]):
                    results.append(False)
                else:
                    results.append(True)
            return results
        if not isinstance(definition, dict) or not len(definition):
            return False
        return all(recurse(definition))

    ### PUBLIC PROPERTIES ###

    @property
    def default_definition(self):
        r'''The default search tree definition, based on the search tree given
        by Paul Nauert:

        ::

            >>> import pprint
            >>> search_tree = quantizationtools.UnweightedSearchTree()
            >>> pprint.pprint(search_tree.default_definition)
            {2: {2: {2: {2: None}, 3: None}, 3: None, 5: None, 7: None},
             3: {2: {2: None}, 3: None, 5: None},
             5: {2: None, 3: None},
             7: {2: None},
             11: None,
             13: None}

        Returns dictionary.
        '''
        return {
            2: {                  # 1/2
                2: {              # 1/4
                    2: {          # 1/8
                        2: None,  # 1/16
                        },
                    3: None,      # 1/12
                    },
                3: None,          # 1/6
                5: None,          # 1/10
                7: None,          # 1/14
                },
            3: {                  # 1/3
                2: {              # 1/6
                    2: None,      # 1/12
                    },
                3: None,          # 1/9
                5: None,          # 1/15
                },
            5: {                  # 1/5
                2: None,          # 1/10
                3: None,          # 1/15
                },
            7: {                  # 1/7
                2: None,          # 1/14
                },
            11: None,             # 1/11
            13: None,             # 1/13
            }



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_Articulation_new_01():
    old_articulation = Articulation('.')
    new_articulation = new(old_articulation)
    assert repr(new_articulation) == "Articulation('.')"



# -*- coding: utf-8 -*-
from abjad.tools import mathtools
from abjad.tools import sequencetools
from abjad.tools.abctools import AbjadValueObject


class InciseSpecifier(AbjadValueObject):
    r'''Incise specifier.

    ..  container:: example

        Specifies one sixteenth rest cut out of the beginning of every
        division:

        ::

            >>> specifier = rhythmmakertools.InciseSpecifier(
            ...     prefix_talea=[-1],
            ...     prefix_counts=[1],
            ...     talea_denominator=16,
            ...     )

    ..  container:: example

        Specifies sixteenth rests cut out of the beginning and end of each
        division:

        ::

            >>> specifier = rhythmmakertools.InciseSpecifier(
            ...     prefix_talea=[-1],
            ...     prefix_counts=[1],
            ...     suffix_talea=[-1],
            ...     suffix_counts=[1],
            ...     talea_denominator=16,
            ...     )

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Specifiers'

    __slots__ = (
        '_body_ratio',
        '_fill_with_notes',
        '_outer_divisions_only',
        '_prefix_counts',
        '_prefix_talea',
        '_suffix_counts',
        '_suffix_talea',
        '_talea_denominator',
        )

    _publish_storage_format = True

    ### INITIALIZER ###

    def __init__(
        self,
        prefix_talea=None,
        prefix_counts=None,
        suffix_talea=None,
        suffix_counts=None,
        talea_denominator=None,
        body_ratio=None,
        fill_with_notes=True,
        outer_divisions_only=None,
        ):
        prefix_talea = prefix_talea or ()
        prefix_talea = tuple(prefix_talea)
        assert self._is_integer_tuple(prefix_talea)
        self._prefix_talea = prefix_talea
        prefix_counts = prefix_counts or ()
        prefix_counts = tuple(prefix_counts)
        assert self._is_length_tuple(prefix_counts)
        self._prefix_counts = prefix_counts
        if prefix_counts and prefix_counts != (0,):
            assert prefix_talea
        if prefix_talea:
            assert prefix_counts
        suffix_talea = suffix_talea or ()
        suffix_talea = tuple(suffix_talea)
        assert self._is_integer_tuple(suffix_talea)
        self._suffix_talea = suffix_talea
        assert self._is_length_tuple(suffix_counts)
        suffix_counts = suffix_counts or ()
        suffix_counts = tuple(suffix_counts)
        self._suffix_counts = suffix_counts
        if suffix_counts and suffix_counts != (0,):
            assert suffix_talea
        if suffix_talea:
            assert suffix_counts
        if talea_denominator is not None:
            if not mathtools.is_nonnegative_integer_power_of_two(
                talea_denominator):
                message = 'talea denominator {!r} must be nonnegative'
                message += ' integer power of 2.'
                message = message.format(talea_denominator)
                raise Exception(message)
        self._talea_denominator = talea_denominator
        if prefix_talea or suffix_talea:
            assert talea_denominator is not None
        if body_ratio is not None:
            body_ratio = mathtools.Ratio(body_ratio)
        self._body_ratio = body_ratio
        assert isinstance(fill_with_notes, bool)
        self._fill_with_notes = fill_with_notes
        assert isinstance(outer_divisions_only, (bool, type(None)))
        self._outer_divisions_only = outer_divisions_only

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats incise specifier.

        ..  container:: example

            Formats incise specifier:

            ::

                >>> specifier = rhythmmakertools.InciseSpecifier(
                ...     prefix_talea=[-1],
                ...     prefix_counts=[1],
                ...     talea_denominator=16,
                ...     )

            ::

                >>> f(specifier)
                rhythmmakertools.InciseSpecifier(
                    prefix_talea=[-1],
                    prefix_counts=[1],
                    talea_denominator=16,
                    )

        ..  container:: example

            Formats incise specifier:

            ::

                >>> specifier = rhythmmakertools.InciseSpecifier(
                ...     prefix_talea=[-1],
                ...     prefix_counts=[0, 1],
                ...     suffix_talea=[-1],
                ...     suffix_counts=[1],
                ...     talea_denominator=16,
                ...     )

            ::

                >>> f(specifier)
                rhythmmakertools.InciseSpecifier(
                    prefix_talea=[-1],
                    prefix_counts=[0, 1],
                    suffix_talea=[-1],
                    suffix_counts=[1],
                    talea_denominator=16,
                    )

        Returns string.
        '''
        superclass = super(InciseSpecifier, self)
        return superclass.__format__(format_specification=format_specification)

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        from abjad.tools import systemtools
        agent = systemtools.StorageFormatAgent(self)
        names = list(agent.signature_keyword_names)
        for name in names[:]:
            if name == 'talea_denominator':
                continue
            if not getattr(self, name):
                names.remove(name)
        # TODO: keywords defaults checking
        if self.fill_with_notes:
            names.remove('fill_with_notes')
        return systemtools.FormatSpecification(
            client=self,
            storage_format_kwargs_names=names,
            )

    @staticmethod
    def _is_integer_tuple(argument):
        if argument is None:
            return True
        if all(isinstance(x, int) for x in argument):
            return True
        return False

    @staticmethod
    def _is_length_tuple(argument):
        if argument is None:
            return True
        if mathtools.all_are_nonnegative_integer_equivalent_numbers(argument):
            if isinstance(argument, (tuple, list)):
                return True
        return False

    @staticmethod
    def _reverse_tuple(argument):
        if argument is not None:
            return tuple(reversed(argument))

    @staticmethod
    def _rotate_tuple(argument, n):
        if argument is not None:
            return tuple(sequencetools.Sequence(argument).rotate(n=n))

    ### PUBLIC PROPERTIES ###

    @property
    def body_ratio(self):
        r'''Gets body ratio.

        ..  container:: example

            Divides middle part of every division ``1:1``:

            ::

                >>> specifier = rhythmmakertools.InciseSpecifier(
                ...     prefix_talea=[-1],
                ...     prefix_counts=[0, 1],
                ...     suffix_talea=[-1],
                ...     suffix_counts=[1],
                ...     talea_denominator=16,
                ...     body_ratio=mathtools.Ratio((1, 1)),
                ...     )
                >>> rhythm_maker = rhythmmakertools.IncisedRhythmMaker(
                ...     incise_specifier=specifier,
                ...     )

            ::

                >>> divisions = 4 * [(5, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 5/16
                        c'8 [
                        c'8 ]
                        r16
                    }
                    {
                        r16
                        c'16. [
                        c'16. ]
                        r16
                    }
                    {
                        c'8 [
                        c'8 ]
                        r16
                    }
                    {
                        r16
                        c'16. [
                        c'16. ]
                        r16
                    }
                }

        Defaults to none.

        Returns ratio or none.
        '''
        return self._body_ratio

    @property
    def fill_with_notes(self):
        r'''Is true when rhythm-maker should fill divisions with notes.
        Otherwise false.

        ..  todo:: Add examples.

        Defaults to true.

        Set to true or false.

        Returns true or false.
        '''
        return self._fill_with_notes

    @property
    def outer_divisions_only(self):
        r'''Is true when rhythm-maker should incise outer divisions only.
        Is false when rhythm-maker should incise all divisions.

        ..  todo:: Add examples.

        Set to true, false or none.

        Defaults to none.

        Returns true, false or none.
        '''
        return self._outer_divisions_only

    @property
    def prefix_counts(self):
        r'''Gets prefix counts.

        ..  todo:: Add examples.

        Returns tuple or none.
        '''
        if self._prefix_counts:
            return list(self._prefix_counts)

    @property
    def prefix_talea(self):
        r'''Gets prefix talea.

        ..  todo:: Add examples.

        Returns tuple or none.
        '''
        if self._prefix_talea:
            return list(self._prefix_talea)

    @property
    def suffix_counts(self):
        r'''Gets suffix counts.

        ..  todo:: Add examples.

        Returns tuple or none.
        '''
        if self._suffix_counts:
            return list(self._suffix_counts)

    @property
    def suffix_talea(self):
        r'''Gets suffix talea.

        ..  todo:: Add examples.

        Returns tuple or none.
        '''
        if self._suffix_talea:
            return list(self._suffix_talea)

    @property
    def talea_denominator(self):
        r'''Gets talea denominator.

        ..  todo:: Add examples.

        Returns positive integer-equivalent number.
        '''
        return self._talea_denominator



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_rhythmtreetools_RhythmTreeContainer_index_01():

    leaf_a = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=3)
    leaf_b = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=3)
    leaf_c = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=2)
    subcontainer = rhythmtreetools.RhythmTreeContainer(
        preprolated_duration=2, children=[leaf_b, leaf_c])
    leaf_d = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=1)
    container = rhythmtreetools.RhythmTreeContainer(
        preprolated_duration=1, children=[leaf_a, subcontainer, leaf_d])

    assert container.index(leaf_a) == 0
    assert container.index(subcontainer) == 1
    assert container.index(leaf_d) == 2

    pytest.raises(ValueError, 'container.index(leaf_b)')
    pytest.raises(ValueError, 'container.index(leaf_c)')



# -*- coding: utf-8 -*-
import abc
from abjad.tools import durationtools
from abjad.tools import systemtools
from abjad.tools.abctools import AbjadObject


class QEvent(AbjadObject):
    r'''Abstract base class from which concrete ``QEvent`` subclasses
    inherit.

    Represents an attack point to be quantized.

    All ``QEvents`` possess a rational offset in milliseconds,
    and an optional index for disambiguating events which fall
    on the same offset in a ``QGrid``.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_index',
        '_offset',
        )

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self, offset=0, index=None):
        offset = durationtools.Offset(offset)
        self._offset = offset
        self._index = index

    ### SPECIAL METHODS ###

    def __lt__(self, argument):
        r'''Is true when `epxr` is a q-event with offset greater than that of this
        q-event. Otherwise false.

        Returns true or false.
        '''
        if type(self) == type(self):
            if self.offset < argument.offset:
                return True
        return False

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        agent = systemtools.StorageFormatAgent(self)
        names = agent.signature_keyword_names
        for name in ('attachments',):
            if not getattr(self, name, None) and name in names:
                names.remove(name)
        return systemtools.FormatSpecification(
            client=self,
            repr_is_indented=False,
            storage_format_kwargs_names=names,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def index(self):
        r'''The optional index, for sorting QEvents with identical offsets.
        '''
        return self._index

    @property
    def offset(self):
        r'''The offset in milliseconds of the event.
        '''
        return self._offset



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Container___init___01():
    r'''Initialize empty container.
    '''

    container = Container([])

    r'''
    {
    }
    '''

    assert isinstance(container, Container)
    assert format(container) == stringtools.normalize(
        r'''
        {
        }
        '''
        )


def test_scoretools_Container___init___02():
    r'''Initialize container with LilyPond note-entry string.
    '''

    container = Container("c'8 d'8 e'8")

    r'''
    {
        c'8
        d'8
        e'8
    }
    '''

    assert isinstance(container, Container)
    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8
            d'8
            e'8
        }
        '''
        )


def test_scoretools_Container___init___03():
    r'''Initialize container with RTM-syntax string.
    '''

    container = Container('rtm: (1 (1 1 1)) (2 (2 (1 (1 1 1)) 2))')

    r'''
    {
        \times 2/3 {
            c'8
            c'8
            c'8
        }
        \times 4/5 {
            c'4
            \times 2/3 {
                c'16
                c'16
                c'16
            }
            c'4
        }
    }
    '''

    assert isinstance(container, Container)
    assert format(container) == stringtools.normalize(
        r'''
        {
            \times 2/3 {
                c'8
                c'8
                c'8
            }
            \times 4/5 {
                c'4
                \times 2/3 {
                    c'16
                    c'16
                    c'16
                }
                c'4
            }
        }
        '''
        )

def test_scoretools_Container___init___04():
    r'''Initialize container with "reduced ly" syntax string.
    '''

    container = Container('abj: 2/3 { 8 8 8 }')

    r'''
    \times 2/3 {
        c'8
        c'8
        c'8
    }
    '''

    assert isinstance(container, Container)
    assert format(container) == stringtools.normalize(
        r'''
        {
            \times 2/3 {
                c'8
                c'8
                c'8
            }
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *
from abjad.demos import part


def test_demos_part_shadow_pitch_contour_reservoir_01():

    pitch_contour_reservoir = part.create_pitch_contour_reservoir()
    shadowed_reservoir = part.shadow_pitch_contour_reservoir(
        pitch_contour_reservoir)

    assert sorted(pitch_contour_reservoir.keys()) == \
        sorted(shadowed_reservoir.keys())

    for key in pitch_contour_reservoir:
        assert isinstance(shadowed_reservoir[key], tuple)
        assert len(pitch_contour_reservoir[key]) == \
            len(shadowed_reservoir[key])



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_indicatortools_Clef___init___01():

    clef_1 = Clef('treble')
    clef_2 = Clef(clef_1)

    assert clef_1 == clef_2
    assert not clef_1 is clef_2


def test_indicatortools_Clef___init___02():

    assert pytest.raises(TypeError, 'Clef(1)')



# -*- coding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools.lilypondparsertools.Music import Music


class SequentialMusic(Music):
    r'''Abjad model of the LilyPond AST sequential music node.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### PUBLIC METHODS ###

    def construct(self):
        r'''Constructs sequential music.

        Returns Abjad container.
        '''
        from abjad.tools import lilypondparsertools
        container = scoretools.Container([])
        for x in self.music:
            if isinstance(x, scoretools.Component):
                container.append(x)
            elif isinstance(x, type(self)):
                container.extend(x.construct())
        return container



# -*- coding: utf-8 -*-
import platform
import textwrap
import unittest
from abjad.tools import abjadbooktools
from abjad.tools import stringtools


@unittest.skipIf(platform.system() == 'Windows', 'No "echo" on Windows')
class ShellDirectiveTests(unittest.TestCase):

    def setUp(self):
        self.handler = abjadbooktools.SphinxDocumentHandler

    def test_1(self):
        source = textwrap.dedent('''
        ..  shell::

            echo "foo"
        ''')
        document = self.handler.parse_rst(source)
        result = stringtools.normalize(document.pformat())
        expected = stringtools.normalize(
            r'''
            <document source="test">
                <literal_block language="console" xml:space="preserve">
                    abjad$ echo "foo"
                    foo
            ''')
        self.assertEqual(result, expected)



# -*- coding: utf-8 -*-
import re
from abjad.tools import mathtools
from abjad.tools import systemtools
from abjad.tools.abctools import AbjadValueObject


class Accidental(AbjadValueObject):
    '''Accidental.

    ..  container:: example

        Sharp:

        ::

            >>> pitchtools.Accidental('s')
            Accidental('s')

    ..  container:: example

        Quarter-sharp:

        ::

            >>> pitchtools.Accidental('qs')
            Accidental('qs')

    ..  container:: example

        Three-quarters-flat:

        ::

            >>> pitchtools.Accidental('tqf')
            Accidental('tqf')

    ..  container:: example

        Three-quarters-sharp:

        ::

            >>> pitchtools.Accidental('#+')
            Accidental('tqs')

    ..  container:: example

        Flat:

        ::

            >>> pitchtools.Accidental('flat')
            Accidental('f')

    ..  container:: example

        Double-sharp:

        ::

            >>> pitchtools.Accidental(2)
            Accidental('ss')

    ..  container:: example

        Four-and-a-half-sharps:

        ::

            >>> pitchtools.Accidental('ssssqs')
            Accidental('ssssqs')

    '''

    ### CLASS VARIABLES ###

    _abbreviation_to_name = {
        'ss': 'double sharp',
        'tqs': 'three-quarters sharp',
        's': 'sharp',
        'qs': 'quarter-sharp',
        '': 'natural',
        'qf': 'quarter-flat',
        'f': 'flat',
        'tqf': 'three-quarters flat',
        'ff': 'double flat',
        }

    _abbreviation_to_semitones = {
        'ff': -2,
        'tqf': -1.5,
        'f': -1,
        'qf': -0.5,
        '': 0,
        'qs': 0.5,
        's': 1,
        'tqs': 1.5,
        'ss': 2,
        }

    _abbreviation_to_symbolic_string = {
        'ff': 'bb',
        'tqf': 'b~',
        'f': 'b',
        'qf': '~',
        '': '',
        'qs': '+',
        's': '#',
        'tqs': '#+',
        'ss': '##',
        }

    _alphabetic_accidental_regex_body = """
        (?P<alphabetic_accidental>
        [s]*(qs)?
        |[f]*(qf)?
        |t?q?[fs]
        |)
        """

    _alphabetic_accidental_regex = re.compile(
        '^{}$'.format(_alphabetic_accidental_regex_body),
        re.VERBOSE,
        )

    _name_to_abbreviation = {
        'double sharp': 'ss',
        'three-quarters sharp': 'tqs',
        'sharp': 's',
        'quarter sharp': 'qs',
        'natural': '',
        'quarter flat': 'qf',
        'flat': 'f',
        'three-quarters flat': 'tqf',
        'double flat': 'ff',
        }

    _semitones_to_abbreviation = {
        -2: 'ff',
        -1.5: 'tqf',
        -1: 'f',
        -0.5: 'qf',
        0: '',
        0.5: 'qs',
        1: 's',
        1.5: 'tqs',
        2: 'ss',
        }

    _symbolic_string_regex_body = '''
        (?P<symbolic_string>
        [#]+[+]?
        |[b]+[~]?
        |[+]
        |[~]
        |
        )
        '''

    _symbolic_string_regex = re.compile(
        '^{}$'.format(_symbolic_string_regex_body),
        re.VERBOSE,
        )

    _symbolic_string_to_abbreviation = {
        'bb': 'ff',
        'b~': 'tqf',
        'b': 'f',
        '~': 'qf',
        '': '',
        '!': '!',
        '+': 'qs',
        '#': 's',
        '#+': 'tqs',
        '##': 'ss',
        }

    _symbolic_string_to_semitones = {
        'bb': -2,
        'b~': -1.5,
        'b': -1,
        '~': -0.5,
        '': 0,
        '+': 0.5,
        '#': 1,
        '#+': 1.5,
        '##': 2,
        }

    __slots__ = (
        '_semitones',
        )

    ### INITIALIZER ##

    def __init__(self, argument=None):
        if argument is None:
            semitones = 0
        elif isinstance(argument, str):
            semitones = 0
            if self.is_abbreviation(argument):
                if argument in self._abbreviation_to_semitones:
                    semitones = self._abbreviation_to_semitones[argument]
                else:
                    while argument and argument.startswith(('f', 's')):
                        if argument[0] == 's':
                            semitones += 1
                        else:
                            semitones -= 1
                        argument = argument[1:]
                    if argument == 'qs':
                        semitones += 0.5
                    elif argument == 'qf':
                        semitones -= 0.5
            elif self.is_symbolic_string(argument):
                if argument in self._symbolic_string_to_semitones:
                    semitones = self._symbolic_string_to_semitones[argument]
                else:
                    while argument and argument.startswith(('b', '#')):
                        if argument[0] == '#':
                            semitones += 1
                        else:
                            semitones -= 1
                        argument = argument[1:]
                    if argument == '+':
                        semitones += 0.5
                    elif argument == '~':
                        semitones -= 0.5
            elif argument in self._name_to_abbreviation:
                abbreviation = self._name_to_abbreviation[argument]
                semitones = self._abbreviation_to_semitones[abbreviation]
            else:
                message = 'can not initialize accidental from value: {!r}'
                message = message.format(argument)
                raise ValueError(message)
        elif isinstance(argument, type(self)):
            semitones = argument.semitones
        elif isinstance(argument, (int, float)):
            semitones = float(argument)
            assert (semitones % 1.) in (0., 0.5)
        elif hasattr(argument, 'accidental'):
            semitones = argument.accidental.semitones
        else:
            message = 'can not initialize accidental from value: {!r}'
            message = message.format(argument)
            raise ValueError(message)
        semitones = mathtools.integer_equivalent_number_to_integer(semitones)
        self._semitones = semitones

    ### SPECIAL METHODS ###

    def __add__(self, argument):
        r'''Adds `argument` to accidental.

        Returns new accidental.
        '''
        if not isinstance(argument, type(self)):
            message = 'can only add accidental to other accidental.'
            raise TypeError(message)
        semitones = self.semitones + argument.semitones
        return type(self)(semitones)

    def __ge__(self, argument):
        r'''Is true when `argument` is an accidental with semitones less than or equal
        to those of this accidental. Otherwise false.

        Returns true or false.
        '''
        return self.semitones >= argument.semitones

    def __gt__(self, argument):
        r'''Is true when `argument` is an accidental with semitones less than
        those of this accidental. Otherwise false.

        Returns true or false.
        '''
        return self.semitones > argument.semitones

    def __le__(self, argument):
        r'''Is true when `argument` is an accidental with semitones greater than or
        equal to those of this accidental. Otherwise false.

        Returns true or false.
        '''
        return self.semitones <= argument.semitones

    def __lt__(self, argument):
        r'''Is true when `argument` is an accidental with semitones greater than those
        of this accidental. Otherwise false.

        Returns true or false.
        '''
        return self.semitones < argument.semitones

    def __ne__(self, argument):
        r'''Is true when accidental does not equal `argument`. Otherwise false.

        Returns true or false.
        '''
        return not self == argument

    def __neg__(self):
        r'''Negates accidental.

        Returns new accidental.
        '''
        return type(self)(-self.semitones)

    def __nonzero__(self):
        r'''Defined equal to true.

        Returns true.
        '''
        return True

    def __str__(self):
        r'''String representation of accidental.

        Returns string.
        '''
        return self.abbreviation

    def __sub__(self, argument):
        r'''Subtracts `argument` from accidental.

        Returns new accidental.
        '''
        if not isinstance(argument, type(self)):
            message = 'can only subtract accidental from other accidental.'
            raise TypeError(message)
        semitones = self.semitones - argument.semitones
        return type(self)(semitones)

    ### PRIVATE PROPERTIES ###

    @property
    def _all_accidental_abbreviations(self):
        return list(self._abbreviation_to_symbolic_string.keys())

    @property
    def _all_accidental_names(self):
        return list(self._name_to_abbreviation.keys())

    @property
    def _all_accidental_semitone_values(self):
        return list(self._semitones_to_abbreviation.keys())

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        return systemtools.FormatSpecification(
            client=self,
            repr_is_indented=False,
            storage_format_args_values=[self.abbreviation],
            storage_format_is_indented=False,
            storage_format_kwargs_names=[],
            template_names=['abbreviation'],
            )

    def _get_lilypond_format(self):
        return self._abbreviation

    ### PUBLIC PROPERTIES ###

    @property
    def abbreviation(self):
        r'''Gets abbreviation of accidental.

        ..  container:: example

            ::

                >>> accidental = pitchtools.Accidental('s')
                >>> accidental.abbreviation
                's'

        Returns string.
        '''
        if self.semitones in self._semitones_to_abbreviation:
            return self._semitones_to_abbreviation[self.semitones]
        character = 's'
        if self.semitones < 0:
            character = 'f'
        semitones = abs(self.semitones)
        semitones, remainder = divmod(semitones, 1.0)
        abbreviation = character * int(semitones)
        if remainder:
            abbreviation += 'q{}'.format(character)
        return abbreviation

    @property
    def is_adjusted(self):
        r'''Is true for all accidentals equal to a nonzero number of semitones.
        Otherwise false.

        ..  container:: example

            ::

                >>> accidental = pitchtools.Accidental('s')
                >>> accidental.is_adjusted
                True

        Returns true or false.
        '''
        return self._semitones != 0

    @property
    def name(self):
        r'''Gets name of accidental.

        ..  container:: example

            ::

                >>> accidental = pitchtools.Accidental('s')
                >>> accidental.name
                'sharp'

        Returns string.
        '''
        abbreviation = self._semitones_to_abbreviation[self.semitones]
        name = self._abbreviation_to_name[abbreviation]
        return name

    @property
    def semitones(self):
        r'''Gets semitones of accidental.

        ..  container:: example

            ::

                >>> accidental = pitchtools.Accidental('s')
                >>> accidental.semitones
                1

        Returns number.
        '''
        return self._semitones

    @property
    def symbolic_string(self):
        r'''Gets symbolic string of accidental.

        ..  container:: example

            ::

                >>> accidental = pitchtools.Accidental('s')
                >>> accidental.symbolic_string
                '#'

        Returns string.
        '''
        abbreviation = self._semitones_to_abbreviation[self.semitones]
        symbolic_string = self._abbreviation_to_symbolic_string[abbreviation]
        return symbolic_string

    ### PUBLIC METHODS ###

    @staticmethod
    def is_abbreviation(argument):
        '''Is true when `argument` is an alphabetic accidental abbreviation.
        Otherwise false.

        ..  container:: example

            ::

                >>> pitchtools.Accidental.is_abbreviation('tqs')
                True

        The regex ``^([s]{1,2}|[f]{1,2}|t?q?[fs])!?$`` underlies this
        predicate.

        Returns true or false.
        '''
        if not isinstance(argument, str):
            return False
        return bool(Accidental._alphabetic_accidental_regex.match(argument))

    @staticmethod
    def is_symbolic_string(argument):
        '''Is true when `argument` is a symbolic accidental string.
        Otherwise false.

        ..  container:: example

            ::

                >>> pitchtools.Accidental.is_symbolic_string('#+')
                True

        Empty string returns true.

        The regex ``^([#]{1,2}|[b]{1,2}|[#]?[+]|[b]?[~]|)$`` underlies this
        predicate.

        Returns true or false.
        '''
        if not isinstance(argument, str):
            return False
        return bool(Accidental._symbolic_string_regex.match(argument))



# -*- coding: utf-8 -*-
from abjad import *


def test_datastructuretools_TypedList___format___01():

    list_ = datastructuretools.TypedList()

    assert format(repr(list_)) == stringtools.normalize(
        'TypedList([])',
        )
    assert format(list_) == stringtools.normalize(
        '''
        datastructuretools.TypedList(
            []
            )
        ''',
        )


def test_datastructuretools_TypedList___format___02():

    list_ = datastructuretools.TypedList([
        'foo',
        instrumenttools.Flute(),
        'bar',
        pitchtools.PitchRange('[A0, C8]'),
        'blah',
        ])

    assert format(list_) == stringtools.normalize(
        r'''
        datastructuretools.TypedList(
            [
                'foo',
                instrumenttools.Flute(
                    instrument_name='flute',
                    short_instrument_name='fl.',
                    instrument_name_markup=markuptools.Markup(
                        contents=['Flute'],
                        ),
                    short_instrument_name_markup=markuptools.Markup(
                        contents=['Fl.'],
                        ),
                    allowable_clefs=instrumenttools.ClefList(
                        [
                            indicatortools.Clef(
                                name='treble',
                                ),
                            ]
                        ),
                    pitch_range=pitchtools.PitchRange(
                        range_string='[C4, D7]',
                        ),
                    sounding_pitch_of_written_middle_c=pitchtools.NamedPitch("c'"),
                    ),
                'bar',
                pitchtools.PitchRange(
                    range_string='[A0, C8]',
                    ),
                'blah',
                ]
            )
        '''
        )


def test_datastructuretools_TypedList___format___03():
    r'''Populated list. With keywords.
    '''

    list_ = datastructuretools.TypedList([
        'foo',
        instrumenttools.Flute(),
        'bar',
        pitchtools.PitchRange('[A0, C8]'),
        'blah'],
        )

    assert format(list_) == stringtools.normalize(
        r'''
        datastructuretools.TypedList(
            [
                'foo',
                instrumenttools.Flute(
                    instrument_name='flute',
                    short_instrument_name='fl.',
                    instrument_name_markup=markuptools.Markup(
                        contents=['Flute'],
                        ),
                    short_instrument_name_markup=markuptools.Markup(
                        contents=['Fl.'],
                        ),
                    allowable_clefs=instrumenttools.ClefList(
                        [
                            indicatortools.Clef(
                                name='treble',
                                ),
                            ]
                        ),
                    pitch_range=pitchtools.PitchRange(
                        range_string='[C4, D7]',
                        ),
                    sounding_pitch_of_written_middle_c=pitchtools.NamedPitch("c'"),
                    ),
                'bar',
                pitchtools.PitchRange(
                    range_string='[A0, C8]',
                    ),
                'blah',
                ]
            )
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_NoteHeadList___getitem___01():
    '''Gets note-head from chord.
    '''

    chord = Chord("<ef' cs'' f''>4")

    assert chord.note_heads[0] is chord.note_heads[0]
    assert chord.note_heads[1] is chord.note_heads[1]
    assert chord.note_heads[2] is chord.note_heads[2]



# -*- coding: utf-8 -*-
from abjad.tools.spannertools.Beam import Beam


class MultipartBeam(Beam):
    r'''Multipart beam.

    Beams together everything that can be beamed and ignores everything else.

    ..  container:: example

        ::

            >>> staff = Staff("c'8 d'8 e'4 f'8 g'8 r4")
            >>> beam = spannertools.MultipartBeam()
            >>> attach(beam, staff[:])
            >>> set_(staff).auto_beaming = False
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff \with {
                autoBeaming = ##f
            } {
                c'8 [
                d'8 ]
                e'4
                f'8 [
                g'8 ]
                r4
            }

    ..  container:: example

        ::

            >>> staff = Staff("c'8 r8 d'8 r8 f'8 g'8 r4")
            >>> beam = spannertools.MultipartBeam()
            >>> attach(beam, staff[:])
            >>> set_(staff).auto_beaming = False
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff \with {
                autoBeaming = ##f
            } {
                c'8
                r8
                d'8
                r8
                f'8 [
                g'8 ]
                r4
            }

    Avoids rests.

    Avoids large-duration notes.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_beam_rests',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        beam_rests=False,
        direction=None,
        overrides=None,
        ):
        Beam.__init__(
            self,
            direction=direction,
            overrides=overrides,
            )
        self._beam_rests = bool(beam_rests)

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        self._beam_rests = self.beam_rests

    def _get_lilypond_format_bundle(self, leaf):
        lilypond_format_bundle = self._get_basic_lilypond_format_bundle(leaf)
        if not self._is_beamable(leaf, beam_rests=self.beam_rests):
            return lilypond_format_bundle
        direction_string = ''
        if self.direction is not None:
            direction_string = '{} '.format(self.direction)
        beamable_leaf_count = 0
        for leaf_ in self._get_leaves():
            if self._is_beamable(leaf_, beam_rests=self.beam_rests):
                beamable_leaf_count += 1
        if 2 <= beamable_leaf_count:
            previous_leaf = leaf._get_leaf(-1)
            if previous_leaf not in self._get_leaves():
                previous_leaf = None
            next_leaf = leaf._get_leaf(1)
            if next_leaf not in self._get_leaves():
                next_leaf = None
            start_piece = None
            stop_piece = None
            if self._is_my_first_leaf(leaf):
                if next_leaf is not None:
                    if self._is_beamable(
                        next_leaf,
                        beam_rests=self.beam_rests,
                        ):
                        start_piece = '{}['.format(direction_string)
            else:
                if previous_leaf is not None:
                    if not self._is_beamable(
                        previous_leaf,
                        beam_rests=self.beam_rests,
                        ):
                        if self._is_beamable(
                            next_leaf,
                            beam_rests=self.beam_rests,
                            ):
                            start_piece = '{}['.format(direction_string)
            if self._is_my_last_leaf(leaf):
                if previous_leaf is not None:
                    if self._is_beamable(
                        previous_leaf,
                        beam_rests=self.beam_rests,
                        ):
                        stop_piece = ']'
            else:
                if self._is_beamable(
                    previous_leaf,
                    beam_rests=self.beam_rests,
                    ):
                    next_leaf = leaf._get_leaf(1)
                    if next_leaf is not None:
                        if not self._is_beamable(
                            next_leaf,
                            beam_rests=self.beam_rests,
                            ):
                            stop_piece = ']'
            if start_piece and stop_piece:
                lilypond_format_bundle.right.spanner_starts.extend([
                    start_piece, stop_piece])
            elif start_piece:
                lilypond_format_bundle.right.spanner_starts.append(start_piece)
            elif stop_piece:
                lilypond_format_bundle.right.spanner_stops.append(stop_piece)
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def beam_rests(self):
        r'''Is true when beam should include rests. Otherwise false.

        ..  container:: example

            Without beamed rests:

            ::

                >>> staff = Staff("c'8 d'8 r8 f'8 g'8 r4.")
                >>> set_(staff).auto_beaming = False
                >>> beam = spannertools.MultipartBeam()
                >>> attach(beam, staff[:])
                >>> show(staff) # doctest: +SKIP

            ::

                >>> f(staff)
                \new Staff \with {
                    autoBeaming = ##f
                } {
                    c'8 [
                    d'8 ]
                    r8
                    f'8 [
                    g'8 ]
                    r4.
                }

            Abjad beams no rests.

            LilyPond beams no rests.

        ..  container:: example

            With beamed rests:

            ::

                >>> staff = Staff("c'8 d'8 r8 f'8 g'8 r4.")
                >>> set_(staff).auto_beaming = False
                >>> beam = spannertools.MultipartBeam(beam_rests=True)
                >>> attach(beam, staff[:])
                >>> show(staff) # doctest: +SKIP

            ::

                >>> f(staff)
                \new Staff \with {
                    autoBeaming = ##f
                } {
                    c'8 [
                    d'8
                    r8
                    f'8
                    g'8
                    r4. ]
                }

            Abjad beams all rests.
            
            LilyPond beams only small-duration rests.

        ..  container:: example

            With beamed rests:

            ::

                >>> staff = Staff("c'8 d'8 r4. f'8 g'8 r8")
                >>> set_(staff).auto_beaming = False
                >>> beam = spannertools.MultipartBeam(beam_rests=True)
                >>> attach(beam, staff[:])
                >>> show(staff) # doctest: +SKIP

            ::

                >>> f(staff)
                \new Staff \with {
                    autoBeaming = ##f
                } {
                    c'8 [
                    d'8
                    r4.
                    f'8
                    g'8
                    r8 ]
                }

            Abjad beams all rests.

            LilyPond beams only small-duration rests.

        ..  container:: example

            With beamed skips:

            ::

                >>> staff = Staff("c'8 d'8 s4. f'8 g'8 s8")
                >>> set_(staff).auto_beaming = False
                >>> beam = spannertools.MultipartBeam(beam_rests=True)
                >>> attach(beam, staff[:])
                >>> show(staff) # doctest: +SKIP

            ::

                >>> f(staff)
                \new Staff \with {
                    autoBeaming = ##f
                } {
                    c'8 [
                    d'8
                    s4.
                    f'8
                    g'8
                    s8 ]
                }

            Abjad beams all skips.

            LilyPond beams all (internal) skips.

        Defaults to false.

        Set to true or false.

        Returns true of false.
        '''
        return self._beam_rests



# -*- coding: utf-8 -*-
from abjad.tools import mathtools
from abjad.tools.quantizationtools.SearchTree import SearchTree


class WeightedSearchTree(SearchTree):
    r'''A search tree that allows for dividing nodes in a ``QGrid`` into
    parts with unequal weights:

    ::

        >>> search_tree = quantizationtools.WeightedSearchTree()

    ::

        >>> print(format(search_tree))
        quantizationtools.WeightedSearchTree(
            definition={
                'divisors': (2, 3, 5, 7),
                'max_depth': 3,
                'max_divisions': 2,
                },
            )

    In ``WeightedSearchTree``'s definition:

        * ``divisors`` controls the sum of the parts of the ratio a node
          may be divided into,
        * ``max_depth`` controls how many levels of tuplet nesting
          are permitted, and
        * ``max_divisions`` controls the maximum permitted length of the
          weights in the ratio.

    Thus, the default ``WeightedSearchTree`` permits the following ratios:

    ::

        >>> for x in search_tree.all_compositions:
        ...     x
        ...
        (1, 1)
        (2, 1)
        (1, 2)
        (4, 1)
        (3, 2)
        (2, 3)
        (1, 4)
        (6, 1)
        (5, 2)
        (4, 3)
        (3, 4)
        (2, 5)
        (1, 6)

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_all_compositions',
        '_compositions',
        '_definition',
        )

    ### INITIALIZER ###

    def __init__(self, definition=None):
        SearchTree.__init__(self, definition)
        self._compositions = self._precompute_compositions()
        all_compositions = []
        for value in list(self._compositions.values()):
            all_compositions.extend(value)
        self._all_compositions = tuple(all_compositions)

    ### PRIVATE METHODS ###

    def _find_leaf_subdivisions(self, parentage_ratios):
        if len(parentage_ratios[1:]) < self._definition['max_depth']:
            return self._all_compositions
        return ()

    def _is_valid_definition(self, definition):
        if not isinstance(definition, dict):
            return False
        elif 'divisors' not in definition:
            return False
        elif not len(definition['divisors']):
            return False
        elif not all(isinstance(x, int) and
            1 < x for x in definition['divisors']):
            return False
        elif not all(mathtools.divisors(x) == [1, x]
            for x in definition['divisors']):
            return False
        elif 'max_depth' not in definition:
            return False
        elif not isinstance(definition['max_depth'], int):
            return False
        elif not 0 < definition['max_depth']:
            return False
        elif 'max_divisions' not in definition:
            return False
        elif not isinstance(definition['max_divisions'], int):
            return False
        elif not 1 < definition['max_divisions']:
            return False
        return True

    def _precompute_compositions(self):
        compositions = {}
        max_divisions = self._definition['max_divisions']
        for divisor in self._definition['divisors']:
            compositions[divisor] = [tuple(x) for x in
                mathtools.yield_all_compositions_of_integer(divisor)
                if 1 < len(x) <= max_divisions]
        return compositions

    ### PUBLIC PROPERTIES ###

    @property
    def all_compositions(self):
        r'''All compositions of weighted search tree.
        '''
        return self._all_compositions

    @property
    def default_definition(self):
        r'''Default definition of weighted search tree.

        Returns dictionary.
        '''
        return {
            'divisors': (2, 3, 5, 7),
            'max_depth': 3,
            'max_divisions': 2,
            }
